@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2017
+/* Copyright (c) 1997-2018
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -33,6 +33,40 @@
 #include <forward_list>
 
 namespace pm {
+
+
+template <typename Coefficient, typename Exponent>
+class UniPolynomial;
+template <typename Coefficient, typename Exponent>
+class Polynomial;
+template <typename Coefficient, typename Exponent>
+class RationalFunction;
+
+// some magic needed for operator construction
+struct is_polynomial {};
+
+// forward declarations needed for friends
+template <typename Coefficient, typename Exponent>
+Div< UniPolynomial<Coefficient, Exponent> >
+div(const UniPolynomial<Coefficient, Exponent>& num, const UniPolynomial<Coefficient, Exponent>& den);
+
+template <typename Coefficient, typename Exponent>
+UniPolynomial<Coefficient, Exponent>
+gcd(const UniPolynomial<Coefficient, Exponent>& a, const UniPolynomial<Coefficient, Exponent>& b);
+
+template <typename Coefficient, typename Exponent>
+ExtGCD< UniPolynomial<Coefficient, Exponent> >
+ext_gcd(const UniPolynomial<Coefficient, Exponent>& a, const UniPolynomial<Coefficient, Exponent>& b,
+        bool normalize_gcd=true);
+
+template <typename Coefficient, typename Exponent>
+UniPolynomial<Coefficient, Exponent>
+lcm(const UniPolynomial<Coefficient, Exponent>& a, const UniPolynomial<Coefficient, Exponent>& b);
+
+template <typename T, typename std::enable_if<std::is_same<typename object_traits<T>::generic_tag,is_polynomial>::value, int>::type=0>
+T pow(const T& base, long exp);
+
+
 namespace polynomial_impl {
 
 // how to convert something to a coefficient of a Polynomial ---------------------
@@ -53,7 +87,7 @@ template <typename T, typename Coefficient>
 struct deeper_coefficient_impl<T, Coefficient,
                                typename std::enable_if<std::is_convertible<T, typename Coefficient::coefficient_type>::value>::type>
 : std::true_type {
-  typedef Coefficient coefficient_type;
+  using coefficient_type = Coefficient;
 
   static coefficient_type construct(const T& x, const int n_vars)
   {
@@ -67,8 +101,8 @@ struct deeper_coefficient_impl<T, Coefficient,
                                typename std::enable_if<!std::is_convertible<T, typename Coefficient::coefficient_type>::value &&
                                                        deeper_coefficient_impl<T, typename Coefficient::coefficient_type>::value>::type>
 : std::true_type {
-  typedef deeper_coefficient_impl<T, typename Coefficient::coefficient_type> deeper;
-  typedef Coefficient coefficient_type;
+  using deeper = deeper_coefficient_impl<T, typename Coefficient::coefficient_type>;
+  using coefficient_type = Coefficient;
 
   static coefficient_type construct(const T& x, const int n_vars)
   {
@@ -128,7 +162,7 @@ class cmp_monomial_ordered
   : public cmp_monomial_ordered_base<Order, strict>
 {
 public:
-  typedef Order exponent_type;
+  using exponent_type = Order;
 
   explicit cmp_monomial_ordered(const exponent_type& order_arg)
     : order(order_arg) {}
@@ -190,17 +224,20 @@ struct nesting_level : int_constant<0> {};
 
 template <typename Exponent>
 struct UnivariateMonomial {
-  typedef Exponent exponent_type;
-  typedef Exponent monomial_type;
-  typedef Vector<Exponent> monomial_list_type;
-  typedef MultivariateMonomial<Exponent> homogenized_type;
+  template <typename exp>
+  using new_type = UnivariateMonomial<exp>;
+  using exponent_type = Exponent;
+  using monomial_type = Exponent;
+  using monomial_list_type = Vector<Exponent>;
+  using homogenized_type = MultivariateMonomial<Exponent>;
   static Exponent deg(const monomial_type& m) { return m; }
   static monomial_type default_value(const int n_vars) { return zero_value<Exponent>(); }
   static bool equals_to_default(const monomial_type& m) { return is_zero(m); }
 
-  static monomial_type empty_value(const int n_vars)
+  static monomial_type empty_value(const int n_vars, int sign = 1)
   {
-    return std::numeric_limits<Exponent>::has_infinity ? - std::numeric_limits<Exponent>::infinity() : std::numeric_limits<Exponent>::min();
+    return std::numeric_limits<Exponent>::has_infinity ? -sign * std::numeric_limits<Exponent>::infinity() : 
+                                          sign > 0 ? std::numeric_limits<Exponent>::min() : std::numeric_limits<Exponent>::max();
   }
   static void croak_if_incompatible(const monomial_type& m, const int n_vars)
   {
@@ -237,17 +274,20 @@ struct UnivariateMonomial {
 
 template <typename Exponent>
 struct MultivariateMonomial {
-  typedef Exponent exponent_type;
-  typedef SparseVector<Exponent> monomial_type;
-  typedef Matrix<Exponent> monomial_list_type;
-  typedef MultivariateMonomial<Exponent> homogenized_type;
+  template <typename exp>
+  using new_type = MultivariateMonomial<exp>;
+  using exponent_type = Exponent;
+  using monomial_type = SparseVector<Exponent>;
+  using monomial_list_type = Matrix<Exponent>;
+  using homogenized_type = MultivariateMonomial<Exponent>;
   static Exponent deg(const monomial_type& m) { return accumulate(m, operations::add<Exponent, Exponent>()); }
   static monomial_type default_value(const int n_vars) { return monomial_type(n_vars); }
   static bool equals_to_default(const monomial_type& m) { return m.empty(); }
 
-  static monomial_type empty_value(const int n_vars)
+  static monomial_type empty_value(const int n_vars, int sign = 1)
   { 
-    return same_element_vector<Exponent>((std::numeric_limits<Exponent>::has_infinity ? - std::numeric_limits<Exponent>::infinity() : std::numeric_limits<Exponent>::min()),
+    return same_element_vector<Exponent>((std::numeric_limits<Exponent>::has_infinity ? -sign * std::numeric_limits<Exponent>::infinity() : 
+                                          sign > 0 ? std::numeric_limits<Exponent>::min() : std::numeric_limits<Exponent>::max() ),
                                          n_vars);
   }
 
@@ -314,14 +354,30 @@ template <typename Monomial, typename Coefficient>
 class GenericImpl {
 
   template <typename> friend struct pm::spec_object_traits;
+  template <typename, typename> friend class GenericImpl;
 
 public:
-  typedef Coefficient coefficient_type;
-  typedef typename Monomial::exponent_type exponent_type;
-  typedef typename Monomial::monomial_type monomial_type;
-  typedef typename std::forward_list<monomial_type> sorted_terms_type;
-  typedef hash_map<monomial_type, coefficient_type> term_hash;
-  typedef typename Monomial::monomial_list_type monomial_list_type;
+  using coefficient_type = Coefficient;
+  using exponent_type = typename Monomial::exponent_type;
+  using monomial_type = typename Monomial::monomial_type;
+  using sorted_terms_type = typename std::forward_list<monomial_type>;
+  using term_hash = hash_map<monomial_type, coefficient_type>;
+  using monomial_list_type = typename Monomial::monomial_list_type;
+
+
+  friend
+     Div< UniPolynomial<Coefficient, exponent_type> >
+     div<>(const UniPolynomial<Coefficient, exponent_type>& num, const UniPolynomial<Coefficient, exponent_type>& den);
+  friend
+     UniPolynomial<Coefficient, exponent_type>
+     gcd<>(const UniPolynomial<Coefficient, exponent_type>& a, const UniPolynomial<Coefficient, exponent_type>& b);
+  friend
+     ExtGCD< UniPolynomial<Coefficient, exponent_type> >
+     ext_gcd<>(const UniPolynomial<Coefficient, exponent_type>& a, const UniPolynomial<Coefficient, exponent_type>& b,
+           bool normalize_gcd);
+  friend
+     UniPolynomial<Coefficient, exponent_type>
+     lcm<>(const UniPolynomial<Coefficient, exponent_type>& a, const UniPolynomial<Coefficient, exponent_type>& b);
 
   static constexpr int coefficient_nesting_level=nesting_level<coefficient_type>::value;
 
@@ -333,9 +389,7 @@ public:
   struct fits_as_coefficient
     : bool_constant<can_upgrade<T, coefficient_type>::value || is_deeper_coefficient<T>::value> {};
 
-  GenericImpl() {}
-
-  explicit GenericImpl(const int n_vars)
+  explicit GenericImpl(const int n_vars = 0)
     : n_variables(n_vars)
     , the_sorted_terms_set(false) {}
 
@@ -363,11 +417,14 @@ public:
       add_term(*m, *c, std::false_type());
   }
 
-  GenericImpl(const int n_vars, const term_hash& src)
+  GenericImpl(const term_hash& src, const int n_vars)
     : n_variables(n_vars)
     , the_terms(src)
     , the_sorted_terms_set(false) {}
                
+  GenericImpl(const int n_vars, const term_hash& src)
+    : GenericImpl(src,n_vars) {}
+
   // Can't have this as constructor, might be ambiguous if coefficient_type = int
   static GenericImpl fromMonomial(const monomial_type&m, const coefficient_type& c, const int n_vars)
   {
@@ -395,7 +452,7 @@ public:
     if (n_vars() != other.n_vars()) throw std::runtime_error("Polynomials of different rings");
   }
 
-  int n_vars() const { return n_variables; }
+  const int& n_vars() const { return n_variables; }
 
   int n_terms() const { return the_terms.size(); }
 
@@ -404,11 +461,11 @@ public:
 					
   bool trivial() const { return the_terms.empty(); }
 
-  bool unit() const
+  bool is_one() const
   {
     return the_terms.size()==1
         && Monomial::equals_to_default(the_terms.begin()->first)
-        && is_one(the_terms.begin()->second);
+        && pm::is_one(the_terms.begin()->second);
   }
 
   Vector<coefficient_type> coefficients_as_vector() const
@@ -461,7 +518,7 @@ public:
 
   typename Monomial::exponent_type lower_deg() const
   {
-    typename Monomial::exponent_type low= Monomial::deg( - Monomial::empty_value(n_variables));
+    typename Monomial::exponent_type low= Monomial::deg( Monomial::empty_value(n_variables, -1 ));
     for (auto it=entire(get_terms()); !it.at_end(); ++it)
       assign_min(low, Monomial::deg(it->first));
     return low;
@@ -636,6 +693,20 @@ public:
     return *this;
   }
 
+  GenericImpl div_exact(const GenericImpl& b)
+  {
+    GenericImpl quot;
+    remainder(b, quot);
+    return quot;
+  }
+
+  GenericImpl& operator%= (const GenericImpl& b)
+  {
+      quot_black_hole black_hole;
+      remainder(b, black_hole);
+      return *this;
+  }
+
   //! Divide by the coefficient of the leading monomial
   GenericImpl& normalize()
   {
@@ -674,15 +745,28 @@ public:
   }
 
   template <typename E>
-  GenericImpl exponentiate_monomial(const E& exp) const
+  typename std::enable_if<std::numeric_limits<E>::is_integer, GenericImpl>::type
+  exponentiate_monomial(const E& exp) const
   {
     if (the_terms.size() != 1)
-      throw std::runtime_error("Except for positive integers, Exponentiation is only implemented for normalized monomials");
+      throw std::runtime_error("exponentiate_monomial: invalid term number");
+    const auto& t = *(the_terms.begin());
+    GenericImpl result(n_variables);
+    result.the_terms.emplace(monomial_type(t.first * exp), pm::pow(t.second,exp));
+    return result;
+  }
+
+  template <typename E>
+  typename std::enable_if<!std::numeric_limits<E>::is_integer, GenericImpl>::type
+  exponentiate_monomial(const E& exp) const
+  {
+    if (the_terms.size() != 1)
+      throw std::runtime_error("exponentiate_monomial: invalid term number");
     const auto& t = *(the_terms.begin());
     if (t.second != one_value<coefficient_type>())
-      throw std::runtime_error("Except for positive integers, Exponentiation is only implemented for normalized monomials");
+      throw std::runtime_error("Except for integers, Exponentiation is only implemented for normalized monomials");
     GenericImpl result(n_variables);
-    result.the_terms.emplace(t.first * exp, t.second);
+    result.the_terms.emplace(monomial_type(t.first * exp), t.second);
     return result;
   }
 
@@ -702,7 +786,7 @@ public:
 
     GenericImpl result(one_value<coefficient_type>(), n_variables);
     if (exp != 0) {
-      int e=exp;
+      long e=exp;
       GenericImpl pow2(*this);
       for (;;) {
         if (e & 1) {
@@ -718,6 +802,16 @@ public:
     return result;
   }
 
+  template <typename Exp=exponent_type, typename T>
+  auto substitute_monomial(const T& exponent) const
+  {
+    typedef typename Monomial::template new_type<Exp> mon;
+    GenericImpl<mon,Coefficient> tmp(n_variables);
+    for (const auto& term : the_terms)
+      tmp.the_terms.emplace(typename mon::monomial_type(convert_to<Exp>(term.first * exponent)), term.second);
+    return tmp;
+  }
+
   GenericImpl operator- () const
   {
     GenericImpl result(n_variables, the_terms);
@@ -841,7 +935,7 @@ public:
   template <typename Output> static
   void pretty_print_term(Output& out, const monomial_type& m, const coefficient_type& c)
   {
-    if (!is_one(c)) {
+    if (!pm::is_one(c)) {
       if (is_minus_one(c)) {
         out << "- ";
       } else {
@@ -928,27 +1022,6 @@ public:
     return compare_ordered(p, cmp_monomial_ordered_base<exponent_type>());
   }
 
-  friend
-  bool operator< (const GenericImpl& p1, const GenericImpl& p2)
-  {
-    return p1.compare(p2) == cmp_lt;
-  }
-  friend
-  bool operator> (const GenericImpl& p1, const GenericImpl& p2)
-  {
-    return p1.compare(p2) == cmp_gt;
-  }
-  friend
-  bool operator<= (const GenericImpl& p1, const GenericImpl& p2)
-  {
-    return p1.compare(p2) != cmp_gt;
-  }
-  friend
-  bool operator>= (const GenericImpl& p1, const GenericImpl& p2)
-  {
-    return p1.compare(p2) != cmp_lt;
-  }
-
   size_t get_hash() const noexcept
   {
     return hash_func<int>()(n_variables) * hash_func<term_hash>()(the_terms);
@@ -1011,6 +1084,57 @@ public:
     return lt_it;
   }
 
+private:
+  struct quot_black_hole
+  {
+    template <bool trusted>
+    void add_term(const monomial_type&, const Coefficient&, bool_constant<trusted>) const {}
+  };
+
+    // replace this with a remainder of division by b, consume the quotient
+  // data must be brought in exclusive posession before calling this method.
+  template <typename QuotConsumer>
+  void remainder(const GenericImpl& b, QuotConsumer& quot_consumer)
+  {
+    const auto b_lead=b.find_lex_lm();
+    typename term_hash::const_iterator this_lead;
+
+    while ((this_lead=find_lex_lm()) != get_mutable_terms().cend() && this_lead->first >= b_lead->first) {
+      const Coefficient k = this_lead->second / b_lead->second;
+      const monomial_type d = this_lead->first - b_lead->first;
+      quot_consumer.add_term(d, k, std::false_type());
+      forget_sorted_terms();
+
+      for (const auto& b_term : b.get_terms()) {
+        auto it = get_mutable_terms().find_or_insert(b_term.first + d);
+        if (it.second) {
+          it.first->second= -k * b_term.second;
+        } else if (is_zero(it.first->second -= k * b_term.second)) {
+          get_mutable_terms().erase(it.first);
+        }
+      }
+    }
+  }
+
+  // replace this with a remainder of division by b, consume the quotient
+  // data must be brought in exclusive posession before calling this method.
+  template <typename QuotConsumer>
+  void remainder(const monomial_type& b, QuotConsumer& quot_consumer)
+  {
+    for (auto it=the_terms.begin(), end=the_terms.end();  it != end;  ) {
+      if (it->first < b) {
+        ++it;
+      } else {
+        if (!std::is_same<QuotConsumer, quot_black_hole>::value)
+          quot_consumer.add_term(it->first - b, it->second, std::false_type());
+        the_terms.erase(it++);
+      }
+    }
+    forget_sorted_terms();
+  }
+
+private:
+
   template <typename T, bool trusted>
   void add_term(const monomial_type& m, T&& c, bool_constant<trusted>)
   {
@@ -1058,57 +1182,14 @@ protected:
   mutable bool the_sorted_terms_set;
 };
 
+
+template <typename Monomial, typename Coefficient>
+struct impl_chooser {
+  typedef GenericImpl< Monomial, Coefficient> type;
+};
+
 } //end namespace polynomial_impl 
 
-
-template <typename Coefficient, typename Exponent>
-struct spec_object_traits< Serialized< polynomial_impl::GenericImpl<polynomial_impl::UnivariateMonomial<Exponent>, Coefficient> > >
-  : spec_object_traits<is_composite> {
-
-  typedef polynomial_impl::UnivariateMonomial<Exponent> Monomial;
-  typedef polynomial_impl::GenericImpl<Monomial,Coefficient> masquerade_for;
-
-  typedef typename polynomial_impl::GenericImpl<Monomial,Coefficient>::term_hash elements;
-
-  template <typename Me, typename Visitor>
-  static void visit_elements(Me& me, Visitor& v)
-  {
-    // here we read a serialized polynomial and should clear the sorted terms first
-    me.forget_sorted_terms();
-    v << me.the_terms;
-    me.n_variables = 1;
-  }
-
-  template <typename Me, typename Visitor>
-  static void visit_elements(const Me& me, Visitor& v)
-  {
-    v << me.the_terms;
-  }
-};
-
-template <typename Coefficient, typename Exponent>
-struct spec_object_traits< Serialized< polynomial_impl::GenericImpl<polynomial_impl::MultivariateMonomial<Exponent>, Coefficient> > >
-  : spec_object_traits<is_composite> {
-
-  typedef polynomial_impl::MultivariateMonomial<Exponent> Monomial;
-  typedef polynomial_impl::GenericImpl<Monomial,Coefficient> masquerade_for;
-  typedef cons<typename polynomial_impl::GenericImpl<Monomial,Coefficient>::term_hash, int> elements;
-
-  template <typename Me, typename Visitor>
-  static void visit_elements(Me& me, Visitor& v)
-  {
-    // here we read a serialized polynomial and should clear the sorted terms first
-    me.forget_sorted_terms();
-    v << me.the_terms << me.n_variables;
-  }
-
-  template <typename Me, typename Visitor>
-  static void visit_elements(const Me& me, Visitor& v)
-  {
-    v << me.the_terms << me.n_variables;
-  }
-};
-
 } //end namespace pm
 
 #endif // POLYMAKE_POLYNOMIALIMPL_H
