@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2018
+/* Copyright (c) 1997-2019
    Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
    http://www.polymake.org
 
@@ -14,83 +14,12 @@
 --------------------------------------------------------------------------------
 */
 
-#include "polymake/client.h"
-#include "polymake/Rational.h"
-#include "polymake/Matrix.h"
-#include "polymake/ListMatrix.h"
-#include "polymake/IncidenceMatrix.h"
-#include "polymake/PowerSet.h"
-#include "polymake/linalg.h"
-#include "polymake/Array.h"
-#include "polymake/common/labels.h"
+#include "polymake/polytope/projection.h"
 
 namespace polymake { namespace polytope {
 
 namespace {
 
-template<typename Scalar>
-Set<int> coordinates_to_eliminate(const Array<int>& indices, int first_coord, int last_coord, int codim, const Matrix<Scalar>& linear_span, bool revert)
-{
-   Set<int> coords_to_eliminate;
-   if (indices.empty()) {
-      bool found=false;
-      for (auto i=entire(all_subsets_of_k(range(first_coord, last_coord), codim)); !found && !i.at_end(); ++i) {
-         if (det(linear_span.minor(All,*i))!=0) {
-            coords_to_eliminate=*i;
-            found=true;
-         }
-      }
-      if (!found) throw std::runtime_error("projection: no non-singular minor in LINEAR_SPAN!");
-   } else {
-      for (auto i=entire(indices); !i.at_end(); ++i) {
-         if (*i<first_coord || *i>last_coord)
-            throw std::runtime_error("projection: index out of range");
-         coords_to_eliminate+=*i;
-      }
-      if (!revert)
-         coords_to_eliminate=range(first_coord,last_coord)-coords_to_eliminate;
-   }
-   return coords_to_eliminate;
-}
-
-template<typename Scalar>
-void process_rays(perl::Object& p_in, int first_coord, const Array<int>& indices, perl::OptionSet& options, const Matrix<Scalar>& linear_span, const Set<int>& coords_to_eliminate, perl::Object& p_out)
-{
-   Matrix<Scalar> Rays, lineality;
-   bool points_read=false;
-   std::string got_property;
-  
-   if (p_in.lookup_with_property_name("RAYS | INPUT_RAYS", got_property) >> Rays) {
-      points_read=true;
-      if ( indices.empty() )  { // if we do a full projection then vertices remain vertices, so we write back whatever we got
-         p_out.take(got_property) << Rays.minor(All,~coords_to_eliminate);
-         if (p_in.lookup("LINEALITY_SPACE | INPUT_LINEALITY") >> lineality && lineality.rows() > 0)
-            p_out.take( got_property=="RAYS" || got_property=="VERTICES" ? "LINEALITY_SPACE" : "INPUT_LINEALITY") << lineality.minor(All,~coords_to_eliminate);
-         else {
-            Matrix<Rational> empty(0, Rays.cols() - coords_to_eliminate.size());
-            p_out.take( got_property=="RAYS" || got_property=="VERTICES" ? "LINEALITY_SPACE" : "INPUT_LINEALITY") << empty;
-         }
-      } else {
-         p_out.take("INPUT_RAYS") << remove_zero_rows(Rays.minor(All,~coords_to_eliminate));
-         if (p_in.lookup("LINEALITY_SPACE | INPUT_LINEALITY") >> lineality && lineality.rows() > 0) 
-            p_out.take("INPUT_LINEALITY") << lineality.minor(All,~coords_to_eliminate);  
-         else {
-            Matrix<Rational> empty(0, Rays.cols() - coords_to_eliminate.size());
-            p_out.take("INPUT_LINEALITY") << empty;
-         }
-      }
-   }
-  
-   if (!points_read && options["nofm"])
-      throw std::runtime_error("projection: no rays found and Fourier-Motzkin elimination excluded");
-   if ( indices.empty() && !options["no_labels"]) {
-      // here we assume that, if VERTEX_LABELS are present in the object, then also VERTICES are known
-      // otherwise this will trigger a convex hull computation
-      int n_vertices = p_in.give("N_RAYS");
-      const std::vector<std::string> labels = common::read_labels(p_in, "RAY_LABELS", n_vertices);
-      p_out.take("RAY_LABELS") << labels;
-   }
-}
 
 template<typename Scalar>
 void process_facets(perl::Object& p_in, const Array<int>& indices, perl::OptionSet& options, const Matrix<Scalar>& linear_span, const Set<int>& coords_to_eliminate, perl::Object& p_out)
@@ -142,40 +71,55 @@ void process_facets(perl::Object& p_in, const Array<int>& indices, perl::OptionS
 }  // end anonymous namespace
 
 template <typename Scalar>
-perl::Object projection_impl(perl::Object p_in, const std::string object_prefix, const std::string linear_span_name, int first_coord, const Array<int> indices, perl::OptionSet options)
+perl::Object projection_cone_impl(perl::Object p_in, const Array<int> indices, perl::OptionSet options)
 {
-   if ( (object_prefix=="CONE" || object_prefix=="FAN") && !p_in.exists("RAYS | INPUT_RAYS") &&
-        ! (object_prefix=="CONE" && p_in.exists("FACETS | INEQUALITIES") ) )
-      throw std::runtime_error("projection is not defined for combinatorially given objects");
+   if ( !p_in.exists("RAYS | INPUT_RAYS") &&
+        !p_in.exists("FACETS | INEQUALITIES") )
+      throw std::runtime_error("projection is not defined for combinatorially given cones");
    
-   const int ambient_dim = p_in.give(object_prefix + "_AMBIENT_DIM");
-   const int dim = p_in.give(object_prefix + "_DIM");
+   int first_coord = p_in.isa("Polytope") ? 1 : 0;
+   const int ambient_dim = p_in.give("CONE_AMBIENT_DIM");
+   const int dim = p_in.give("CONE_DIM");
    const int codim = ambient_dim-dim;
    if (indices.empty() && codim==0) return p_in; // nothing to do
 
-   const Matrix<Scalar> linear_span = p_in.give(linear_span_name);
+   const Matrix<Scalar> linear_span = p_in.give("LINEAR_SPAN");
    if (codim != linear_span.rows())
-      throw std::runtime_error("projection: " + linear_span_name + " has wrong number of rows");
+      throw std::runtime_error("projection: LINEAR_SPAN has wrong number of rows");
    const int last_coord=ambient_dim-1;
    const Set<int> coords_to_eliminate = coordinates_to_eliminate(indices, first_coord, last_coord, codim, linear_span, options["revert"]);   // set of columns to project to
 
    perl::Object p_out(p_in.type());
 
-   if ((object_prefix=="CONE" || object_prefix=="FAN") && p_in.exists("RAYS | INPUT_RAYS"))
+   if (p_in.exists("RAYS | INPUT_RAYS"))
       process_rays(p_in, first_coord, indices, options, linear_span, coords_to_eliminate, p_out);
 
-   if (object_prefix=="VECTOR" && p_in.exists("VECTORS")){
-      const Matrix<Scalar> vec = p_in.give("VECTORS") ;
-      p_out.take("VECTORS") <<  vec.minor(All,~coords_to_eliminate);
-   }
-
-   if (object_prefix=="CONE" && p_in.exists("FACETS | INEQUALITIES"))
+   if (p_in.exists("FACETS | INEQUALITIES"))
       process_facets(p_in, indices, options, linear_span, coords_to_eliminate, p_out);
 
-   if (object_prefix=="FAN") {
-      IncidenceMatrix<> MC;
-      if (p_in.lookup("MAXIMAL_CONES") >> MC)
-         p_out.take("MAXIMAL_CONES") << MC;
+   return p_out;
+}
+
+template <typename Scalar>
+perl::Object projection_vectorconfiguration_impl(perl::Object p_in, const Array<int> indices, perl::OptionSet options)
+{
+   int first_coord = p_in.isa("PointConfiguration") ? 1 : 0;
+   const int ambient_dim = p_in.give("VECTOR_AMBIENT_DIM");
+   const int dim = p_in.give("VECTOR_DIM");
+   const int codim = ambient_dim-dim;
+   if (indices.empty() && codim==0) return p_in; // nothing to do
+
+   const Matrix<Scalar> linear_span = p_in.give("LINEAR_SPAN");
+   if (codim != linear_span.rows())
+      throw std::runtime_error("projection: LINEAR_SPAN has wrong number of rows");
+   const int last_coord=ambient_dim-1;
+   const Set<int> coords_to_eliminate = coordinates_to_eliminate(indices, first_coord, last_coord, codim, linear_span, options["revert"]);   // set of columns to project to
+
+   perl::Object p_out(p_in.type());
+
+   if (p_in.exists("VECTORS")){
+      const Matrix<Scalar> vec = p_in.give("VECTORS") ;
+      p_out.take("VECTORS") <<  vec.minor(All,~coords_to_eliminate);
    }
    return p_out;
 }
@@ -240,7 +184,9 @@ perl::Object projection_preimage_impl(const Array<perl::Object>& pp_in)
    return p_out;
 }
 
-FunctionTemplate4perl("projection_impl<Scalar=Rational>($$$$$ {revert => 0, nofm => 0})");
+FunctionTemplate4perl("projection_cone_impl<Scalar=Rational>(Cone $ {revert => 0, nofm => 0})");
+
+FunctionTemplate4perl("projection_vectorconfiguration_impl<Scalar=Rational>(VectorConfiguration $ {revert => 0, nofm => 0})");
 
 FunctionTemplate4perl("projection_preimage_impl<Scalar=Rational>($)");
 
